@@ -63,6 +63,10 @@ class AWSTaskQueueAPI(object):
   def completed(self):
     raise NotImplementedError()
 
+  @property
+  def leased(self):
+    raise NotImplementedError()
+
   def is_empty():
     return self.enqueued == 0
 
@@ -82,15 +86,18 @@ class AWSTaskQueueAPI(object):
       if len(batch) == 0:
         break
 
-      resp = self.sqs.send_message_batch(
-        QueueUrl=self.qurl,
-        Entries=[ {
+      entries = [ {
           "Id": str(j),
           "MessageBody": json.dumps(task),
           "DelaySeconds": delay_seconds,
-        } for j, task in enumerate(batch) ],
+        } for j, task in enumerate(batch) 
+      ]
+
+      resp = self.sqs.send_message_batch(
+        QueueUrl=self.qurl,
+        Entries=entries,
       )
-      total += 1
+      total += len(entries)
 
     return total
 
@@ -156,7 +163,7 @@ class AWSTaskQueueAPI(object):
       pass
 
   def tally(self):
-    raise NotImplementedError("Not supported for SQS.")
+    pass
 
   def purge(self):
     # This is more efficient, but it kept freezing
@@ -170,9 +177,6 @@ class AWSTaskQueueAPI(object):
       for task in tasks:
         self.delete(task)
     return self
-
-  def list(self):
-    return list(self)
     
   def __iter__(self):
     return iter(self._request(num_tasks=10, visibility_timeout=0))
