@@ -12,7 +12,6 @@ import time
 
 import tenacity
 
-from .registered_task import totask, totaskid
 from .lib import mkdir, jsonify, toiter, STRING_TYPES, sip, toabs
 
 retry = tenacity.retry(
@@ -226,7 +225,8 @@ class FileQueueAPI(object):
 
   def add_insert_count(self, ct):
     try:
-      N = int(read_file(self.insertions_path))
+      N = read_file(self.insertions_path) # potential multiprocess race condition
+      N = int(N) if N != '' else 0
     except FileNotFoundError:
       N = 0
 
